;Delay and Mute 20 PCB
;last modified 2023-07-20

; When relay is ON the audio is NOT muted.

;Picaxe 14M2 Pinout
;Pin1  = +V
;Pin2  = C.5 (In) Serial In
;Pin3  = C.4 (Touch/ADC/Out/In)
;Pin4  = C.3 (In)
;Pin5  = C.2 (kb clk/hpwm A/pwm/Out/In)
;Pin6  = C.1 (kb data/hpwm B/Out/In)
;Pin7  = C.0 (hpwm C/pwm/Touch/ADC/Out/In)
;Pin8  = B.5 (In/Out/ADC/Touch/hpwm D)
;Pin9  = B.4 (In/Out/ADC/Touch/pwm/hi2c sda)
;Pin10 = B.3 (In/Out/ADC/Touch/hi2c scl)
;Pin11 = B.2 (In/Out/ADC/Touch/pwm/SRQ)
;Pin12 = B.1 (In/Out/ADC/Touch/SRI/hserin)
;Pin13 = B.0 Serial Out (Out/hserout/DAC)
;Pin14 = 0V

	
#Picaxe 14M2
SETFREQ m4
#No_Data
#Terminal 4800

PAUSE 800


'clear terminal to display progress messages
'Sertxd ($0C) 'clear the terminal screen to display values
'Sertxd ("Starting",13,10)


;Pin Assignments
SYMBOL Delay_Pot   = C.0 'Start Delay (0-60,000 milliseconds 0-60) seconds
SYMBOL PS_Pin      = B.4 ' =<5V PS divider for Mute Control & Power FAIL sensing. 
SYMBOL Percent_Pot = B.5 'Set Percent of PS nominal to Mute (50~100% PS)
SYMBOL Relay_CtrlA = C.2 '0=muted 1=unmuted
SYMBOL Relay_CtrlB = C.1 '0=muted 1=unmuted
SYMBOL LED	    	 = C.4 'Mute LED driver Muted=1 UnMuted=0


;Variables Storage W0=b0,b1 W1=b2,b3 W2=b4,b5
SYMBOL OneTime       = b5 'to limit recalc to once at delayed PS read
SYMBOL LEDstatus     = b4 'track LED ON or OFF	
SYMBOL Delay_Value   = W3 'calculated Startup Delay in milliseconds 
SYMBOL PS_Value      = W4 'measured power supply nominal voltage
SYMBOL Percent_Value = W5 'measured value of percent pot 
SYMBOL PS_MinSet     = W6 'Calculated voltage as percentage of PS (% of normal)
SYMBOL DelayPS2ndRead= W8 'number of seconds after start to recheck PS values

'values for 2nd PS calc delay & execution
OneTime       = 0 ' 0=not read yet 1=already read 2nd time the PS calc
LEDstatus     = 0
Delay_Value   = 0 
PS_Value      = 0 
Percent_Value = 0 
PS_MinSet     = 0 
DelayPS2ndRead   = 15 'in seconds


'if PS good then proceed, otherwise stay in Mute loop
PS_MinSet = 300 '8.9V 
READADC10 PS_Pin, PS_Value
IF PS_Value < PS_MinSet THEN 'voltage is < 9V, normal range 10~30
	GOSUB Mute_Audio 'stay muted until voltage rises > 9
ENDIF


'Ensure Position of Relay at startup
'Sertxd ("==Initializing (cycling) Relay==",13,10)
GOSUB Relay_Ctrl_ON
HIGH LED
PAUSE 30
GOSUB Relay_Ctrl_OFF
LOW LED


'Sertxd ("==Performing Initial PS Mute Volts Calc.==",13,10)
GOSUB Calc_for_Mute


'Sertxd ("==Do Calculation of Start Delay==",13,10)
GOSUB Calc_Start_Delay


'unmute to enable audio output
'Sertxd ("==Unmuting audio output==",13,10)
GOSUB Relay_Ctrl_ON


'Start main program Loop
'Sertxd ("==Starting Main Loop==",13,10)
Main:

'each loop, get current level reading 
READADC10 PS_Pin, PS_Value
'compare current power level to disconnect level and act accordingly
IF PS_Value < PS_MinSet THEN
	GOSUB Mute_Audio
ENDIF

'A one-time Recalc mute threshold at 15 seconds and disable after one read
IF OneTime = 0 THEN
	IF time > DelayPS2ndRead THEN
		'Sertxd ("==2nd (delayed) PS Mute Calculation==",TAB,#time,13,10)
		GOSUB Calc_for_Mute
		OneTime = 1
	ENDIF
ENDIF

'Loop back to Main
GOTO Main

'======================================

'if current voltage is < Mute_Level then mute the audio
Mute_Audio:
'Sertxd ("==PS Below Mute threshold or Mute Switch==",13,10)
GOSUB Relay_Ctrl_OFF
DO WHILE PS_Value < PS_MinSet 'remain here until Current Level rises above Mute Level
	Readadc10 PS_Pin, PS_Value
	IF LEDstatus = 1 THEN
		LOW LED
		LEDstatus = 0
	ELSE
		HIGH LED
		LEDstatus = 1
	ENDIF
PAUSE 600
LOOP
LOW LED
GOSUB Relay_Ctrl_ON:

RETURN


Calc_for_Mute: 'Set Threshold for Mute as Percentage of PS nominal when power fails
'Sertxd ("==Calculate Percentage of PS to use for Mute Level==",13,10)

'read PS voltage nominal value
READADC10 PS_Pin, PS_Value 
'Sertxd ("PS Nominal Count",TAB,#PS_Value,13,10)

'find percent value of nominal to be used to determine cutoff voltage
READADC10 Percent_Pot, Percent_Value
'Sertxd ("Percent Pot Count",TAB,#Percent_Value,13,10)

'convert pot 0~1023 count to 512~1023 limited to 970 (50%~95%)
Percent_Value = Percent_Value / 2 + 512 MAX 972 'max allowed is 95%
'Sertxd ("0-100% becomes 50%-95% range",TAB,#Percent_Value,13,10)

'Calculate Cutoff Value (using percentage and nominal)
Percent_Value = Percent_Value * 64
PS_MinSet = PS_Value ** Percent_Value
'Sertxd ("PS Cutoff converted to 50~95% of PS Nominal",TAB,#PS_MinSet,13,10)

RETURN


Calc_Start_Delay:
'Sertxd ("==Calculating Startup Delay==",13,10)
READADC10 Delay_Pot, Delay_Value
'Sertxd ("Delay_Pot",Tab,#Delay_Value,13,10)
IF Delay_Value < 512 THEN 'first half rotation is 0-10 seconds
	Delay_Value = Delay_Value / 50 MIN 0
ELSEIF Delay_Value > 511 THEN 'second half rotation is 10-60 seconds
	Delay_Value = Delay_Value - 511 / 10 + 10 MAX 60
ENDIF
'Sertxd ("Calculated Delay in seconds",TAB,#Delay_Value,13,10)
'delay # seconds set by Delay Pot
DelayPS2ndRead = DelayPS2ndRead + Delay_Value 'add delay to 2nd PS calc timing
'Sertxd ("Total Delay including 2nd PS Mute Calc",TAB,#DelayPS2ndRead,13,10)
	DO WHILE time < Delay_Value 'wait until # seconds since power on
LOOP 'resume counting time
'Pause for the specified milliseconds
'Sertxd ("Finished Calculated Delay",13,10)

RETURN

'RELAY CONTROL

Relay_Ctrl_OFF: 'Audio Output disable
'troubleshooting terminal data
Input Relay_CtrlA
Input Relay_CtrlB
PAUSE 5
Low Relay_CtrlA
Low Relay_CtrlB
'Sertxd ("Relay_Ctrl_Off (Muted)",13,10)
PAUSE 60
RETURN

Relay_Ctrl_ON: 'Audio Output enable
'troubleshooting terminal data
Input Relay_CtrlA
Input Relay_CtrlB
PAUSE 5
High Relay_CtrlA
High Relay_CtrlB
'Sertxd ("Relay_Ctrl_ON (Unmuted)",13,10)
PAUSE 60
RETURN	

